@@ -128,7 +128,8 @@ static BalsaMailboxConfView *
                                GtkWindow * window,
                                GtkWidget * table, gint row,
                                GtkSizeGroup * size_group,
-                               MailboxConfWindow * mcw);
+                               MailboxConfWindow * mcw,
+                               GCallback callback);
 
 /* pages */
 static GtkWidget *create_dialog(MailboxConfWindow *mcw);
@@ -206,14 +207,22 @@ balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc, LibBalsaServer *s,
         { N_("Required"),    LIBBALSA_TLS_REQUIRED }
     };
     GtkWidget *label;
-    GtkWidget *hbox;
+    GtkWidget *box;
     gboolean use_ssl = s && s->use_ssl;
 
-    hbox = gtk_hbox_new(FALSE, 0);
+    box = gtk_vbox_new(FALSE, 0);
+
+#if !defined(USE_SSL)
+    gtk_box_pack_start(GTK_BOX(box),
+                       gtk_label_new
+                       (_("Balsa was built without SSL support.\n"
+                          "Neither SSL nor TLS can be used.")),
+                       FALSE, FALSE, 0);
+#endif                          /* !defined(USE_SSL) */
 
     bsc->table = GTK_TABLE(libbalsa_create_table(3 + extra_rows, 2));
     gtk_container_set_border_width(GTK_CONTAINER(bsc->table), 12);
-    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(bsc->table),
+    gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(bsc->table),
                        FALSE, FALSE, 0);
 
     bsc->used_rows = 0;
@@ -221,6 +230,9 @@ balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc, LibBalsaServer *s,
     bsc->use_ssl = balsa_server_conf_add_checkbox(bsc, _("Use _SSL"));
     if(use_ssl)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bsc->use_ssl), TRUE);
+#if !defined(USE_SSL)
+    gtk_widget_set_sensitive(bsc->use_ssl, FALSE);
+#endif                          /* !defined(USE_SSL) */
 
     label = libbalsa_create_label(_("Use _TLS:"), GTK_WIDGET(bsc->table), 1);
 
@@ -238,8 +250,12 @@ balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc, LibBalsaServer *s,
     gtk_widget_show_all(GTK_WIDGET(bsc->table));
     bsc->used_rows = 2;
     gtk_widget_set_sensitive(bsc->tls_option, !use_ssl);
+#if !defined(USE_TLS)
+    gtk_widget_set_sensitive(label, FALSE);
+    gtk_widget_set_sensitive(bsc->tls_option, FALSE);
+#endif                          /* !defined(USE_TLS) */
 
-    return hbox;
+    return box;
 }
 
 GtkWidget*
@@ -947,7 +963,6 @@ mailbox_conf_update(MailboxConfWindow *mcw)
             g_free(mailbox->name);
 	    mailbox->name = name;
 	    balsa_mblist_mailbox_node_redraw(mbnode);
-	    balsa_window_update_tab(mbnode);
 	} else
 	    g_free(name);
 
@@ -1118,6 +1133,8 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
     GtkFileChooserAction action;
     GtkWidget *entry = NULL;
     GtkSizeGroup *size_group;
+    const gchar *type;
+    gchar *title;
 
     table = libbalsa_create_table(3, 2);
 
@@ -1130,16 +1147,22 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
                          mcw, row, NULL, label);
     } else mcw->mailbox_name = NULL;
 
+    type = g_type_name(mcw->mailbox_type) + 15;
+    title = g_strdup_printf(mcw->mailbox ?
+                            _("Local %s Mailbox Properties") :
+                            _("New Local %s Mailbox"), type);
+
     action = mcw->mailbox_type == LIBBALSA_TYPE_MAILBOX_MBOX ?
         GTK_FILE_CHOOSER_ACTION_SAVE :
         GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER;
     dialog =
-        gtk_file_chooser_dialog_new(_("Local Mailbox Configurator"),
+        gtk_file_chooser_dialog_new(title,
                                     GTK_WINDOW(balsa_app.main_window),
                                     action,
                                     mcw->ok_button_name, MCW_RESPONSE,
                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                     NULL);
+    g_free(title);
 #if HAVE_MACOSX_DESKTOP
     libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(balsa_app.main_window));
 #endif
@@ -1160,7 +1183,7 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
 
     mcw->view_info =
         mailbox_conf_view_new_full(mcw->mailbox, GTK_WINDOW(dialog), table,
-                                   ++row, size_group, mcw);
+                                   ++row, size_group, mcw, NULL);
 
     return dialog;
 }
@@ -1386,7 +1409,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
 
     mcw->view_info =
         mailbox_conf_view_new_full(mcw->mailbox, GTK_WINDOW(dialog), table,
-                                   ++row, NULL, mcw);
+                                   ++row, NULL, mcw, NULL);
 
     return dialog;
 }
@@ -1420,7 +1443,8 @@ mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
                            GtkWindow * window,
                            GtkWidget * table, gint row,
                            GtkSizeGroup * size_group,
-                           MailboxConfWindow * mcw)
+                           MailboxConfWindow * mcw,
+                           GCallback callback)
 {
     GtkWidget *label;
     BalsaMailboxConfView *view_info;
@@ -1458,6 +1482,9 @@ mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
     }
     gtk_table_attach(GTK_TABLE(table), widget, 1, 2, row, row + 1,
                      GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+    if (callback)
+        g_signal_connect_swapped(view_info->identity_combo_box, "changed",
+                                 callback, window);
 
 #ifdef HAVE_GPGME
     {
@@ -1485,6 +1512,9 @@ mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
         if (mcw)
             g_signal_connect(view_info->chk_crypt, "changed",
                              G_CALLBACK(check_for_blank_fields), mcw);
+        if (callback)
+            g_signal_connect_swapped(view_info->chk_crypt, "changed",
+                                     callback, window);
 	gtk_table_attach(GTK_TABLE(table), view_info->chk_crypt,
 		         1, 2, row, row + 1,
 			 GTK_EXPAND | GTK_FILL, 0, 0, 0);
@@ -1500,6 +1530,9 @@ mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
     if (mcw)
         g_signal_connect(view_info->show_to, "toggled",
                          G_CALLBACK(check_for_blank_fields), mcw);
+    if (callback)
+        g_signal_connect_swapped(view_info->show_to, "toggled",
+                                 callback, window);
     
     /* Subscribe check button */
     view_info->subscribe =
@@ -1510,16 +1543,20 @@ mailbox_conf_view_new_full(LibBalsaMailbox * mailbox,
     if (mcw)
         g_signal_connect(view_info->subscribe, "toggled",
                          G_CALLBACK(check_for_blank_fields), mcw);
+    if (callback)
+        g_signal_connect_swapped(view_info->subscribe, "toggled",
+                                 callback, window);
 
     return view_info;
 }
 
 BalsaMailboxConfView *
 mailbox_conf_view_new(LibBalsaMailbox * mailbox,
-                      GtkWindow * window, GtkWidget * table, gint row)
+                      GtkWindow * window, GtkWidget * table, gint row,
+                      GCallback callback)
 {
     return mailbox_conf_view_new_full(mailbox, window, table, row,
-                                      NULL, NULL);
+                                      NULL, NULL, callback);
 }
 
 #ifdef HAVE_GPGME
