@@ -168,13 +168,25 @@ load_toolbars(void)
 #endif /* ENABLE_TOUCH_UI */
 #define TRASH_NAME "Trash"
 
+static void
+sr_special_notify(gpointer data, GObject * mailbox)
+{
+    LibBalsaMailbox **special = data;
+
+    if (special == &balsa_app.trash && !balsa_app.mblist_tree_store
+        && balsa_app.empty_trash_on_exit)
+        empty_trash(balsa_app.main_window);
+
+    *special = NULL;
+}
+
 static gchar *specialNames[] = {
     INBOX_NAME, SENTBOX_NAME, TRASH_NAME, DRAFTS_NAME, OUTBOX_NAME
 };
 void
 config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
 {
-    LibBalsaMailbox **special;
+    LibBalsaMailbox **special, *old_mailbox;
     BalsaMailboxNode *mbnode;
 
     g_return_if_fail(mailbox != NULL);
@@ -195,26 +207,35 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
     default:
 	return;
     }
-    if (*special) {
-	g_free((*special)->config_prefix); 
-	(*special)->config_prefix = NULL;
-        if (!LIBBALSA_IS_MAILBOX_LOCAL(*special)
-            || !libbalsa_path_is_below_dir(libbalsa_mailbox_local_get_path
-                                           (*special),
-                                           balsa_app.local_mail_directory))
-            config_mailbox_add(*special, NULL);
-	g_object_remove_weak_pointer(G_OBJECT(*special), (gpointer) special);
+    if ((old_mailbox = *special)) {
+        *special = NULL;
+        g_free(old_mailbox->config_prefix);
+        old_mailbox->config_prefix = NULL;
 
-	mbnode = balsa_find_mailbox(*special);
-	*special = NULL;
+        g_free(old_mailbox->name);
+        old_mailbox->name = g_path_get_basename(old_mailbox->url);
+
+        if (!LIBBALSA_IS_MAILBOX_LOCAL(old_mailbox)
+            || !libbalsa_path_is_below_dir(libbalsa_mailbox_local_get_path
+                                           (old_mailbox),
+                                           balsa_app.local_mail_directory))
+            config_mailbox_add(old_mailbox, NULL);
+
+        g_object_weak_unref(G_OBJECT(old_mailbox),
+                            (GWeakNotify) sr_special_notify, special);
+
+        mbnode = balsa_find_mailbox(old_mailbox);
 	balsa_mblist_mailbox_node_redraw(mbnode);
 	g_object_unref(mbnode);
     }
     config_mailbox_delete(mailbox);
+    g_free(mailbox->name);
+    mailbox->name = g_strdup(specialNames[which]);
     config_mailbox_add(mailbox, specialNames[which]);
 
     *special = mailbox;
-    g_object_add_weak_pointer(G_OBJECT(*special), (gpointer) special);
+    g_object_weak_ref(G_OBJECT(mailbox), (GWeakNotify) sr_special_notify,
+                      special);
 
     mbnode = balsa_find_mailbox(mailbox);
     balsa_mblist_mailbox_node_redraw(mbnode);
@@ -392,18 +413,6 @@ pop3_progress_notify(LibBalsaMailbox* mailbox, int msg_type, int prog, int tot,
 
 /* Initialize the specified mailbox, creating the internal data
    structures which represent the mailbox. */
-static void
-sr_special_notify(gpointer data, GObject * mailbox)
-{
-    LibBalsaMailbox **special = data;
-
-    if (special == &balsa_app.trash && !balsa_app.mblist_tree_store
-        && balsa_app.empty_trash_on_exit)
-        empty_trash(balsa_app.main_window);
-
-    *special = NULL;
-}
-
 static gint
 config_mailbox_init(const gchar * prefix)
 {
@@ -786,6 +795,7 @@ config_global_load(void)
         balsa_app.layout_type != LAYOUT_WIDE_SCREEN)
         balsa_app.layout_type = LAYOUT_DEFAULT;
     balsa_app.view_message_on_open = libbalsa_conf_get_bool("ViewMessageOnOpen=true");
+    balsa_app.ask_before_select = libbalsa_conf_get_bool("AskBeforeSelect=false");
     balsa_app.pgdownmod = libbalsa_conf_get_bool("PageDownMod=false");
     balsa_app.pgdown_percent = libbalsa_conf_get_int("PageDownPercent=50");
     if (balsa_app.pgdown_percent < 10)
@@ -954,7 +964,7 @@ config_global_load(void)
 	/* Transition code */
 	LibBalsaSmtpServer *smtp_server;
 	LibBalsaServer *server;
-	gchar *passphrase;
+	gchar *passphrase, *hostname;
 
 	smtp_server = libbalsa_smtp_server_new();
 	libbalsa_smtp_server_set_name(smtp_server,
@@ -963,11 +973,11 @@ config_global_load(void)
 	    g_slist_prepend(NULL, smtp_server);
 	server = LIBBALSA_SERVER(smtp_server);
 
-	libbalsa_server_set_host
-	(server,
-	 libbalsa_conf_get_string_with_default("ESMTPServer=localhost:25", 
-					     &def_used),
-	 FALSE);
+        hostname = 
+            libbalsa_conf_get_string_with_default("ESMTPServer=localhost:25", 
+                                                  &def_used);
+	libbalsa_server_set_host(server, hostname, FALSE);
+        g_free(hostname);
 	libbalsa_server_set_username(server,
 		libbalsa_conf_get_string("ESMTPUser"));
 
@@ -994,7 +1004,7 @@ config_global_load(void)
     }
 #endif                          /* ENABLE_ESMTP */
     /* ... outgoing mail */
-    balsa_app.wordwrap = libbalsa_conf_get_bool("WordWrap=true");
+    balsa_app.wordwrap = libbalsa_conf_get_bool("WordWrap=false");
     balsa_app.wraplength = libbalsa_conf_get_int("WrapLength=72");
     if (balsa_app.wraplength < 40)
 	balsa_app.wraplength = 40;
@@ -1299,6 +1309,7 @@ config_save(void)
     libbalsa_conf_set_int("ProgressWindow", balsa_app.pwindow_option);
     libbalsa_conf_set_int("LayoutType",     balsa_app.layout_type);
     libbalsa_conf_set_bool("ViewMessageOnOpen", balsa_app.view_message_on_open);
+    libbalsa_conf_set_bool("AskBeforeSelect", balsa_app.ask_before_select);
     libbalsa_conf_set_bool("PageDownMod", balsa_app.pgdownmod);
     libbalsa_conf_set_int("PageDownPercent", balsa_app.pgdown_percent);
 #if defined(ENABLE_TOUCH_UI)
