@@ -19,15 +19,19 @@
  * 02111-1307, USA.
  */
 
-#include "config.h"
-
-#include <gnome.h>
-#include <string.h>
-
-#include "balsa-app.h"
+#if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
+# include "config.h"
+#endif                          /* HAVE_CONFIG_H */
 #include "store-address.h"
 
-#include "libbalsa.h"
+#include <string.h>
+#include <glib/gi18n.h>
+
+#if HAVE_MACOSX_DESKTOP
+#  include "macosx-helpers.h"
+#endif
+
+#include "balsa-app.h"
 
 /* global data */
 typedef struct _StoreAddressInfo StoreAddressInfo;
@@ -58,16 +62,20 @@ static void store_address_book_menu_cb(GtkWidget * widget,
                                        StoreAddressInfo * info);
 static void store_address_add_address(StoreAddressInfo * info,
                                       const gchar * label,
-                                      LibBalsaAddress * address);
-static void store_address_add_glist(StoreAddressInfo * info,
-                                    const gchar * label, GList * list);
+                                      InternetAddress * address,
+                                      InternetAddress * group);
+static void store_address_add_lbaddress(StoreAddressInfo * info,
+                                        const LibBalsaAddress *address);
+static void store_address_add_list(StoreAddressInfo * info,
+                                   const gchar * label,
+				   InternetAddressList * list);
 
 /* 
  * public interface: balsa_store_address
  */
 #define BALSA_STORE_ADDRESS_KEY "balsa-store-address"
 void
-balsa_store_address(GList * messages)
+balsa_store_address_from_messages(GList * messages)
 {
     StoreAddressInfo *info = NULL;
     GList *message_list = NULL;
@@ -86,7 +94,7 @@ balsa_store_address(GList * messages)
     if (!message_list) {
         /* All messages are already showing. */
         if (info)
-            gdk_window_raise(info->dialog->window);
+            gtk_window_present(GTK_WINDOW(info->dialog));
         return;
     }
 
@@ -97,8 +105,8 @@ balsa_store_address(GList * messages)
     info->dialog = store_address_dialog(info);
 
     if (!info->entries_list) {
-        gnome_appbar_set_status(balsa_app.appbar,
-                                _("Store address: no addresses"));
+        balsa_information(LIBBALSA_INFORMATION_WARNING,
+                          _("Store address: no addresses"));
         store_address_free(info);
         return;
     }
@@ -115,6 +123,24 @@ balsa_store_address(GList * messages)
     gtk_widget_show(GTK_WIDGET(info->dialog));
 }
 
+void
+balsa_store_address(const LibBalsaAddress *address)
+{
+    StoreAddressInfo *info = NULL;
+
+    info = g_new(StoreAddressInfo, 1);
+    info->current_address_book = NULL;
+    info->message_list = NULL;
+    info->entries_list = NULL;
+    info->dialog = store_address_dialog(info);
+
+    store_address_add_lbaddress(info, address);
+
+    g_signal_connect(G_OBJECT(info->dialog), "response",
+                     G_CALLBACK(store_address_response), info);
+    gtk_widget_show_all(GTK_WIDGET(info->dialog));
+}
+
 /* Weak notify that a message was deleted; remove it from our list. */
 static void
 store_address_weak_notify(StoreAddressInfo * info, gpointer message)
@@ -182,9 +208,12 @@ store_address_dialog(StoreAddressInfo * info)
                                     GTK_STOCK_SAVE,  SA_RESPONSE_SAVE,
                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                     NULL);
-    GtkWidget *vbox = GTK_DIALOG(dialog)->vbox;
+    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
     GtkWidget *frame, *label;
 
+#if HAVE_MACOSX_DESKTOP
+    libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(balsa_app.main_window));
+#endif
     frame = store_address_book_frame(info);
     if(g_list_length(balsa_app.address_book_list)>1)
         gtk_widget_show_all(frame);
@@ -235,7 +264,7 @@ store_address_from_entries(GtkWindow *window, StoreAddressInfo * info,
                 msg = _("Unexpected address book error. Report it."); break;
             }
         }
-        balsa_information_parented(window, LIBBALSA_INFORMATION_ERROR, msg);
+        balsa_information_parented(window, LIBBALSA_INFORMATION_ERROR, "%s", msg);
     }
     
     g_object_unref(address);
@@ -249,62 +278,36 @@ store_address_book_frame(StoreAddressInfo * info)
 {
     GList *ab_list;
     GtkWidget *frame = gtk_frame_new(_("Choose Address Book"));
-#if GTK_CHECK_VERSION(2, 4, 0)
     GtkWidget *combo_box;
-#else
-    GtkWidget *ab_option, *menu_item, *ab_menu;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     LibBalsaAddressBook *address_book;
-    guint default_ab_offset = 0;
+    guint default_ab_offset = 0, off;
 
-#if GTK_CHECK_VERSION(2, 4, 0)
     combo_box = gtk_combo_box_new_text();
     g_signal_connect(combo_box, "changed",
                      G_CALLBACK(store_address_book_menu_cb), info);
-#else
-    ab_menu = gtk_menu_new();
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     if (balsa_app.address_book_list) {
 	info->current_address_book = balsa_app.default_address_book;
 
-	
-	for(ab_list = balsa_app.address_book_list;
+	/* NOTE: we have to store the default address book index and
+           call set_active() after all books are added to the list or
+           gtk-2.10.4 will lose the setting. */
+	for(off=0, ab_list = balsa_app.address_book_list;
             ab_list; 
-            ab_list = g_list_next(ab_list)) {
+            off++, ab_list = g_list_next(ab_list)) {
 	    address_book = LIBBALSA_ADDRESS_BOOK(ab_list->data);
 	    if (info->current_address_book == NULL)
 		info->current_address_book = address_book;
 
-#if GTK_CHECK_VERSION(2, 4, 0)
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
                                       address_book->name);
 	    if (address_book == balsa_app.default_address_book)
-		gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
-                                         default_ab_offset);
-#else
-	    menu_item = gtk_menu_item_new_with_label(address_book->name);
-	    gtk_menu_shell_append(GTK_MENU_SHELL(ab_menu), menu_item);
-
-	    g_signal_connect(G_OBJECT(menu_item), "activate",
-			     G_CALLBACK(store_address_book_menu_cb), info);
-            g_object_set_data(G_OBJECT(menu_item),"address-book",
-                              GUINT_TO_POINTER(default_ab_offset));
-
-	    if (address_book == balsa_app.default_address_book)
-		gtk_menu_set_active(GTK_MENU(ab_menu), default_ab_offset);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
-
-	    default_ab_offset++;
-
+                default_ab_offset = off;
 	}
+        gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
+                                 default_ab_offset);
     }
-#if GTK_CHECK_VERSION(2, 4, 0)
     gtk_container_add(GTK_CONTAINER(frame), combo_box);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    ab_option = gtk_option_menu_new();
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(ab_option), ab_menu);
-    gtk_container_add(GTK_CONTAINER(frame), ab_option);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
+
     return frame;
 }
 
@@ -322,14 +325,13 @@ store_address_note_frame(StoreAddressInfo *info)
     gtk_container_set_border_width(GTK_CONTAINER(info->notebook), 5);
     gtk_container_add(GTK_CONTAINER(frame), info->notebook);
 
-    for (list = info->message_list; list; list = g_list_next(list)) {
+    for (list = info->message_list; list; list = list->next) {
         message = LIBBALSA_MESSAGE(list->data);
 	if (message->headers) {
-	    if (message->headers->from)
-		store_address_add_address(info, _("From:"), message->headers->from);
-	    store_address_add_glist(info, _("To:"), message->headers->to_list);
-	    store_address_add_glist(info, _("Cc:"), message->headers->cc_list);
-	    store_address_add_glist(info, _("Bcc:"), message->headers->bcc_list);
+	    store_address_add_list(info, _("From:"), message->headers->from);
+	    store_address_add_list(info, _("To:"), message->headers->to_list);
+	    store_address_add_list(info, _("Cc:"), message->headers->cc_list);
+	    store_address_add_list(info, _("Bcc:"), message->headers->bcc_list);
 	}
     }
 
@@ -342,13 +344,7 @@ static void
 store_address_book_menu_cb(GtkWidget * widget, 
                            StoreAddressInfo * info)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     guint i = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    guint i =
-        GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(widget),
-                                           "address-book"));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     GList *nth = g_list_nth(balsa_app.address_book_list, i);
     if(nth)
         info->current_address_book = LIBBALSA_ADDRESS_BOOK(nth->data);
@@ -358,20 +354,28 @@ store_address_book_menu_cb(GtkWidget * widget,
  * make a new page in the notebook */
 static void
 store_address_add_address(StoreAddressInfo * info,
-                          const gchar * lab, LibBalsaAddress * address)
+                          const gchar * lab, InternetAddress * ia,
+			  InternetAddress * group)
 {
     gchar *text;
+    LibBalsaAddress *address;
     gchar *label_text;
     GtkWidget **entries, *ew;
 
-    if (address == NULL)
+    if (ia == NULL)
         return;
 
     entries = g_new(GtkWidget *, NUM_FIELDS);
     info->entries_list = g_list_append(info->entries_list, entries);
-    ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
 
-    text = libbalsa_address_to_gchar(address, 0);
+    text = internet_address_to_string(ia, FALSE);
+    address = libbalsa_address_new();
+    address->full_name =
+        g_strdup(ia->name ? ia->name : group ? group->name : NULL);
+    address->address_list = g_list_prepend(NULL, g_strdup(INTERNET_ADDRESS_MAILBOX (ia)->addr));
+    ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
+    g_object_unref(address);
+
     label_text = g_strconcat(lab, text, NULL);
     g_free(text);
     if (g_utf8_strlen(label_text, -1) > 10)
@@ -382,15 +386,57 @@ store_address_add_address(StoreAddressInfo * info,
     g_free(label_text);
 }
 
-/* store_address_add_glist:
- * take a GList of addresses and pass them one at a time to
+static void
+store_address_add_lbaddress(StoreAddressInfo * info,
+                            const LibBalsaAddress *address)
+{
+    gchar *label_text;
+    GtkWidget **entries, *ew;
+
+    g_return_if_fail(address->address_list);
+    entries = g_new(GtkWidget *, NUM_FIELDS);
+    info->entries_list = g_list_append(info->entries_list, entries);
+
+    ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
+
+    label_text = g_strdup(address->full_name ? address->full_name :
+                          address->address_list->data);
+    if (g_utf8_strlen(label_text, -1) > 10)
+        /* truncate to an arbitrary length: */
+        *g_utf8_offset_to_pointer(label_text, 10) = '\0';
+    gtk_notebook_append_page(GTK_NOTEBOOK(info->notebook), ew,
+                             gtk_label_new(label_text));
+    g_free(label_text);
+}
+
+/* store_address_add_list:
+ * take a list of addresses and pass them one at a time to
  * store_address_add_address */
 static void
-store_address_add_glist(StoreAddressInfo * info,
-                        const gchar * label, GList * list)
+store_address_add_list(StoreAddressInfo * info,
+                       const gchar * label,
+                       InternetAddressList * list)
 {
-    while (list) {
-        store_address_add_address(info, label, list->data);
-        list = g_list_next(list);
+    int i, j;
+    
+    if (!list)
+        return;
+
+    for (i = 0; i < internet_address_list_length (list); i++) {
+        InternetAddress *ia = internet_address_list_get_address (list, i);
+	
+        if (INTERNET_ADDRESS_IS_MAILBOX (ia))
+            store_address_add_address(info, label, ia, NULL);
+        else {
+            InternetAddressList *members = INTERNET_ADDRESS_GROUP (ia)->members;
+
+            for (j = 0; j < internet_address_list_length (members); j++) {
+                InternetAddress *member_address = internet_address_list_get_address (members, j);
+
+                if (INTERNET_ADDRESS_IS_MAILBOX (member_address))
+                    store_address_add_address(info, label, member_address,
+                                              ia);
+            }
+        }
     }
 }
