@@ -24,12 +24,11 @@
 
 #include <unistd.h>
 
-/* FIXME: mailbox_lock is really an internal libbalsa mutex. */
-extern pthread_mutex_t mailbox_lock;
+/*  allocated in main.c */
+extern pthread_mutex_t checking_mail_lock;
 
 /*  define thread globals */
 extern pthread_t get_mail_thread;
-extern int checking_mail;
 extern int mail_thread_pipes[2];
 extern GIOChannel *mail_thread_msg_send;
 extern GIOChannel *mail_thread_msg_receive;
